
#import "FlutterUIWatcher.h"
#import <FinotesCore/Fn.h>


@interface FlutterUIWatcher()

@property dispatch_queue_t queue;
@property (nonatomic, strong) FlutterMethodChannel* flutterChannel;


@end


@implementation FlutterUIWatcher

@synthesize threshold;
@synthesize flutterLocation;


+ (instancetype)sharedInstance {
    static FlutterUIWatcher *instance;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        instance = [[self alloc] init];
    });
    return instance;
}

- (instancetype)init {
    self = [super init];
    if (self) {
        threshold = 250.0; //Choosing 250 ms as that is the recomended metric for apphangs. https://developer.apple.com/documentation/xcode/improving-app-responsiveness
        _queue = dispatch_queue_create("com.finotes.ios.finotescore", DISPATCH_QUEUE_SERIAL);
        
    }
    return self;
    
}


-(void) setFlutterChannel:(FlutterMethodChannel *)flutterChannel {
    if(_flutterChannel || !flutterChannel) return;
    _flutterChannel = flutterChannel;
}

-(void) enable {
    if(!_flutterChannel) return;
    [self start];
}



#pragma UIthread block detection code


-(void) start {
    @try {
        __typeof__(self) __weak wself = self;

        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, threshold * NSEC_PER_MSEC), _queue, ^{
            NSNumber *time = [NSNumber numberWithDouble:[[NSDate date] timeIntervalSince1970]];
            [wself.flutterChannel invokeMethod:@"isThreadLive" arguments:time result:^(id  _Nullable result) {

                int64_t timeSent  = [time doubleValue] * 1000;
                int64_t timeNow = [[NSDate date] timeIntervalSince1970] * 1000;
                int64_t timeGap = timeNow - timeSent;
                if(timeGap > 250) {
                   [Fn buildAppHangIssue:wself.flutterLocation withThreshold:timeGap];
                }
                [self start];

            }];
        });
    } @catch (NSException *exception) {
    }
}



@end
