/// Singleton class to set Finotes plugin configuration values.
class Observe {
  Map<String, int>? _baseUrlTimeout;
  List<String>? _whiteListedDomains;
  List<String>? _maskedHeaders;

  static Observe? _instance;

  Observe._();

  factory Observe() {
    _instance ??= Observe._();
    return _instance!;
  }

  /// API to set timeout values for your base urls.
  ///
  /// timeout values are set in milliseconds. Values cannot be less than 1000 milliseconds.
  void setBaseUrlTimeouts(Map<String, int> timeoutsForBaseUrls) {
    _baseUrlTimeout = timeoutsForBaseUrls;
  }

  /// API to whitelist domain names for tracking.
  ///
  /// Once set, only domains listed using this API will be tracked for issues by Finotes plugin.
  void setWhiteListedDomain(List<String> domainsWhiteListed) {
    _whiteListedDomains = domainsWhiteListed;
  }

  /// API to mask header fields in issue reports sent to the Finotes dashboard.
  ///
  /// Once set all header keys from request and response headers values with the keys will be masked before issue is reported to dashboard.
  void setMaskedHeaders(List<String> headersMasked) {
    _maskedHeaders = headersMasked;
  }

  /// API to retrive url timeouts set by developer
  Map<String, int>? getBaseUrlTimeouts() {
    return _baseUrlTimeout;
  }

  /// API to retrive list of whitelisted domains set by developer
  List<String>? getWhiteListedDomains() {
    return _whiteListedDomains;
  }

  /// API to retrive list of header keys to be masked.
  List<String>? getMaskedHeaderKeys() {
    return _maskedHeaders;
  }
}
