import 'package:finotescore/src/utils.dart';
import 'package:finotescore/src/platform_interface.dart';
import 'package:flutter/material.dart';
import 'package:stack_trace/stack_trace.dart';

class CrashDetector {
  void processAndReport(String exception, StackTrace? stacktrace,
      {String context = "",
      Iterable<DiagnosticsNode> information = const [],
      bool fatal = false}) {
    String stackTrace = stacktrace == null
        ? StackTrace.current.toString()
        : stacktrace.toString();

    String _information = information.isEmpty
        ? ''
        : (StringBuffer()..writeAll(information, '\n')).toString();
    StringBuffer stackTraceWithInfo = StringBuffer();
    stackTraceWithInfo
        .writeAll({exception, context, _information, stackTrace}, '\n');

    String lineNumber = "0";
    String location = Utils.Fn;
    Trace trace = Trace.parseVM(stackTrace.toString()).terse;
    final _obfuscatedStackTraceLineRegExp =
        RegExp(r'^(\s*#\d{2} abs )([\da-f]+)((?: virt [\da-f]+)?(?: .*)?)$');

    for (final Frame frame in trace.frames) {
      if (frame is UnparsedFrame) {
        if (_obfuscatedStackTraceLineRegExp.hasMatch(frame.member)) {
          location = frame.member.replaceFirstMapped(
              _obfuscatedStackTraceLineRegExp,
              (match) => '${match.group(1)}0${match.group(3)}');
        }
      } else {
        lineNumber = frame.line?.toString() ?? '0';
        final String member = frame.member ?? '<fn>';
        final List<String> members = member.split('.');
        if (members.length > 1) {
          location =
              "${frame.library}.${members.first}.${members.sublist(1).join('.')}";
        } else {
          location = member;
        }
      }
    }

    if (fatal) {
      PlatformInterface.buildCrashIssue(
          lineNumber, location, stackTraceWithInfo.toString(), exception);
    } else {
      PlatformInterface.buildException(
          lineNumber, location, stackTraceWithInfo.toString(), exception);
    }
  }
}
