import 'dart:developer';
import 'dart:io';

import 'package:device_info_plus/device_info_plus.dart';
import 'package:finotescore/src/platform_interface.dart';

class MemoryLeak {
  static MemoryLeak? _instance;

  MemoryLeak._();
  List<WeakReference> weakReferenceList = <WeakReference>[];

  factory MemoryLeak() {
    _instance ??= MemoryLeak._();
    return _instance!;
  }

  Future<void> addObject(WeakReference value) async {
    if (Platform.isAndroid) {
      DeviceInfoPlugin deviceInfo = DeviceInfoPlugin();
      var androidInfo = await deviceInfo.androidInfo;
      if (androidInfo.isPhysicalDevice) {
        _addObjectToList(value);
      }
    } else if (Platform.isIOS) {
      DeviceInfoPlugin deviceInfo = DeviceInfoPlugin();
      var iosInfo = await deviceInfo.iosInfo;
      if (iosInfo.isPhysicalDevice) {
        _addObjectToList(value);
      }
    } else {
      log("Platform not yet supported");
    }
  }

  void _addObjectToList(WeakReference value) {
    if (!weakReferenceList.any(
        (element) => element.target.runtimeType == value.target.runtimeType)) {
      weakReferenceList.add(value);
      _processForLeaks();
    }
  }

  void _processForLeaks() {
    log("weak reference count ${weakReferenceList.length}");
    if (weakReferenceList.length <= 3) return;
    var secondItem = weakReferenceList[1];
    log("weak reference count ${secondItem.target}");
    if (secondItem.target != null) {
      //Raise leak issue.
      log("Memory leak detected");
      PlatformInterface.memoryLeakReported(
          secondItem.target.runtimeType.toString(),
          secondItem.target.toString());
    }
    weakReferenceList.remove(secondItem);
  }
}
