import 'dart:collection';

import 'package:flutter/services.dart';

class PlatformInterface {
  static const MethodChannel _channel = MethodChannel('finotescore');

  static Future<dynamic> nativeChannelHandler(MethodCall methodCall) async {
    switch (methodCall.method) {
      case 'isThreadLive':
        return methodCall.arguments;
      default:
        throw MissingPluginException('notImplemented');
    }
  }

  static void establishNativeConnection() async {
    _channel.setMethodCallHandler(nativeChannelHandler);
    Future.delayed(const Duration(seconds: 5), () {
      _channel.invokeMethod('handlerSetupComplete');
    });
  }

  static void init() {
    _channel.invokeMethod('init');
  }

  static void customIssue(
      String where, String description, String detailedDescription) {
    _channel
        .invokeMethod('customIssue', [where, description, detailedDescription]);
  }

  static void setActivityMarker(String where, String marker) {
    _channel.invokeMethod('setActivityMarker', [where, marker]);
  }

  static void buildCrashIssue(
      String lineNumber, String location, String stackTrace, String exception) {
    _channel.invokeMethod(
        'buildCrashIssue', [lineNumber, location, stackTrace, exception]);
  }

  static void buildException(
      String lineNumber, String location, String stackTrace, String exception) {
    _channel.invokeMethod(
        'buildException', [lineNumber, location, stackTrace, exception]);
  }

  static void mainThreadPing(String location, String? stackTrace) {
    _channel.invokeMethod('mainThreadPing', [location, stackTrace]);
  }

  static void flutterLocation(String location) {
    _channel.invokeMethod('flutterLocation', [location]);
  }

  static void logMe() {
    _channel.invokeMethod('log');
  }

  static void setAPIRequest(HashMap<String, Object> request) {
    _channel.invokeMethod('setAPIRequest', [request]);
  }

  static void setAPIResponse(HashMap request) {
    _channel.invokeMethod('setAPIResponse', [request]);
  }

  static void memoryLeakReported(String location, String locationWithDetails) {
    _channel
        .invokeMethod('memoryLeakReported', [location, locationWithDetails]);
  }
}
